package versioning

import (
	"database/sql/driver"
	"strconv"

	"github.com/pkg/errors"
)

func (v *Version) Scan(value interface{}) error {
	var code uint32

	switch val := value.(type) {
	case []byte:
		c, err := strconv.ParseUint(string(val), 10, 32)
		if err != nil {
			return errors.WithStack(err)
		}
		code = uint32(c)

	case int64:
		code = uint32(val)

	default:
		return errors.Errorf("incompatible version code type - %T", value)
	}

	parsed, err := ParseFromCode(code)
	if err != nil {
		return err
	}

	*v = parsed
	return nil
}

func (v Version) Value() (driver.Value, error) {
	return int64(v.Code()), nil
}
